@@ -23,7 +23,7 @@
  */
 
 /**
- * \file core/bell.c Ring the bell or flash the screen
+ * \file bell.c Ring the bell or flash the screen
  *
  * Sometimes, X programs "ring the bell", whatever that means. Metacity lets
  * the user configure the bell to be audible or visible (aka visual), and
@@ -31,13 +31,13 @@
  * We never get told about audible bells; X handles them just fine by itself.
  *
  * Visual bells come in at meta_bell_notify(), which checks we are actually
- * in visual mode and calls through to meta_bell_visual_notify(). That
+ * in visual mode and calls through to bell_visual_notify(). That
  * function then checks what kind of visual flash you like, and calls either
- * meta_bell_flash_fullscreen()-- which calls meta_bell_flash_screen() to do
- * its work-- or meta_bell_flash_frame(), which flashes the focussed window
- * using meta_bell_flash_window_frame(), unless there is no such window, in
- * which case it flashes the screen instead. meta_bell_flash_window_frame()
- * flashes the frame and calls meta_bell_unflash_frame() as a timeout to
+ * bell_flash_fullscreen()-- which calls bell_flash_screen() to do
+ * its work-- or bell_flash_frame(), which flashes the focussed window
+ * using bell_flash_window_frame(), unless there is no such window, in
+ * which case it flashes the screen instead. bell_flash_window_frame()
+ * flashes the frame and calls bell_unflash_frame() as a timeout to
  * remove the flash.
  *
  * The visual bell was the result of a discussion in Bugzilla here:
@@ -46,14 +46,13 @@
  * Several of the functions in this file are ifdeffed out entirely if we are
  * found not to have the XKB extension, which is required to do these clever
  * things with bells; some others are entirely no-ops in that case.
- *
- * \bug Static functions should not be called meta_*.
  */
 
 #include <config.h>
 #include "bell.h"
 #include "screen-private.h"
 #include "prefs.h"
+#include <canberra-gtk.h>
 
 /**
  * Flashes one entire screen.  This is done by making a window the size of the
@@ -74,7 +73,7 @@
  * \bug This appears to destroy our current XSync status.
  */
 static void
-meta_bell_flash_screen (MetaDisplay *display, 
+bell_flash_screen (MetaDisplay *display, 
 			MetaScreen  *screen)
 {
   Window root = screen->xroot;
@@ -147,7 +146,7 @@ meta_bell_flash_screen (MetaDisplay *display,
  */
 #ifdef HAVE_XKB
 static void
-meta_bell_flash_fullscreen (MetaDisplay *display, 
+bell_flash_fullscreen (MetaDisplay *display, 
 			    XkbAnyEvent *xkb_ev)
 {
   XkbBellNotifyEvent *xkb_bell_ev = (XkbBellNotifyEvent *) xkb_ev;
@@ -158,7 +157,7 @@ meta_bell_flash_fullscreen (MetaDisplay *display,
     {
       screen = meta_display_screen_for_xwindow (display, xkb_bell_ev->window);
       if (screen)
-	meta_bell_flash_screen (display, screen);
+	bell_flash_screen (display, screen);
     }
   else 
     {
@@ -166,7 +165,7 @@ meta_bell_flash_fullscreen (MetaDisplay *display,
       while (screen_list) 
 	{
 	  screen = (MetaScreen *) screen_list->data;
-	  meta_bell_flash_screen (display, screen);
+	  bell_flash_screen (display, screen);
 	  screen_list = screen_list->next;
 	}
     }
@@ -174,7 +173,7 @@ meta_bell_flash_fullscreen (MetaDisplay *display,
 
 /**
  * Makes a frame be not flashed; this is the timeout half of
- * meta_bell_flash_window_frame(). This is done simply by clearing the
+ * bell_flash_window_frame(). This is done simply by clearing the
  * flash flag and queuing a redraw of the frame.
  *
  * If the configure script found we had no XKB, this does not exist.
@@ -183,11 +182,11 @@ meta_bell_flash_fullscreen (MetaDisplay *display,
  *              a callback function.
  * \return Always FALSE, so we don't get called again.
  *
- * \bug This is the parallel to meta_bell_flash_window_frame(), so it should
+ * \bug This is the parallel to bell_flash_window_frame(), so it should
  * really be called meta_bell_unflash_window_frame().
  */
 static gboolean 
-meta_bell_unflash_frame (gpointer data)
+bell_unflash_frame (gpointer data)
 {
   MetaFrame *frame = (MetaFrame *) data;
   frame->is_flashing = 0;
@@ -207,13 +206,13 @@ meta_bell_unflash_frame (gpointer data)
  * \param window  The window to flash
  */
 static void
-meta_bell_flash_window_frame (MetaWindow *window)
+bell_flash_window_frame (MetaWindow *window)
 {
   g_assert (window->frame != NULL);
   window->frame->is_flashing = 1;
   meta_frame_queue_draw (window->frame);
   g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, 100, 
-      meta_bell_unflash_frame, window->frame, NULL);
+      bell_unflash_frame, window->frame, NULL);
 }
 
 /**
@@ -224,8 +223,8 @@ meta_bell_flash_window_frame (MetaWindow *window)
  * \param xkb_ev   The bell event we just received
  */
 static void
-meta_bell_flash_frame (MetaDisplay *display, 
-		       XkbAnyEvent *xkb_ev)
+bell_flash_frame (MetaDisplay *display, 
+		  XkbAnyEvent *xkb_ev)
 {
   XkbBellNotifyEvent *xkb_bell_event = (XkbBellNotifyEvent *) xkb_ev;
   MetaWindow *window;
@@ -238,11 +237,11 @@ meta_bell_flash_frame (MetaDisplay *display,
     }
   if (window)
     {
-      meta_bell_flash_window_frame (window);
+      bell_flash_window_frame (window);
     }
   else /* revert to fullscreen flash if there's no focussed window */
     {
-      meta_bell_flash_fullscreen (display, xkb_ev);
+      bell_flash_fullscreen (display, xkb_ev);
     }
 }
 
@@ -258,16 +257,16 @@ meta_bell_flash_frame (MetaDisplay *display,
  * \bug This should be merged with meta_bell_notify().
  */
 static void
-meta_bell_visual_notify (MetaDisplay *display, 
+bell_visual_notify (MetaDisplay *display, 
 			 XkbAnyEvent *xkb_ev)
 {
   switch (meta_prefs_get_visual_bell_type ()) 
     {
     case META_VISUAL_BELL_FULLSCREEN_FLASH:
-      meta_bell_flash_fullscreen (display, xkb_ev);
+      bell_flash_fullscreen (display, xkb_ev);
       break;
     case META_VISUAL_BELL_FRAME_FLASH:
-      meta_bell_flash_frame (display, xkb_ev); /* does nothing yet */
+      bell_flash_frame (display, xkb_ev); /* does nothing yet */
       break;
     case META_VISUAL_BELL_INVALID:
       /* do nothing */
@@ -275,67 +274,60 @@ meta_bell_visual_notify (MetaDisplay *display,
     }
 }
 
-/**
- * Gives the user some kind of visual bell; in fact, this is our response
- * to any kind of bell request, but we set it up so that we only get
- * notified about visual bells, and X deals with audible ones.
- *
- * If the configure script found we had no XKB, this does not exist.
- *
- * \param display  The display the bell event came in on
- * \param xkb_ev   The bell event we just received
- */
 void
 meta_bell_notify (MetaDisplay *display, 
 		  XkbAnyEvent *xkb_ev)
 {
   /* flash something */
   if (meta_prefs_get_visual_bell ()) 
-    meta_bell_visual_notify (display, xkb_ev);
+    bell_visual_notify (display, xkb_ev);
+
+  if (meta_prefs_bell_is_audible ()) 
+    {
+      ca_proplist *p;
+      XkbBellNotifyEvent *xkb_bell_event = (XkbBellNotifyEvent*) xkb_ev;
+      MetaWindow *window;
+      int res;
+
+      ca_proplist_create (&p);
+      ca_proplist_sets (p, CA_PROP_EVENT_ID, "bell-window-system");
+      ca_proplist_sets (p, CA_PROP_EVENT_DESCRIPTION, _("Bell event"));
+
+      window = meta_display_lookup_x_window (display, xkb_bell_event->window);
+      if (!window && (display->focus_window) && (display->focus_window->frame))
+        window = display->focus_window;
+
+      if (window)
+        { 
+          ca_proplist_sets (p, CA_PROP_WINDOW_NAME, window->title);
+          ca_proplist_setf (p, CA_PROP_WINDOW_X11_XID, "%lu", (unsigned long)window->xwindow);
+          ca_proplist_sets (p, CA_PROP_APPLICATION_NAME, window->res_name);
+          ca_proplist_setf (p, CA_PROP_APPLICATION_PROCESS_ID, "%d", window->net_wm_pid);
+        }
+
+      /* First, we try to play a real sound ... */
+      res = ca_context_play_full (ca_gtk_context_get (), 1, p, NULL, NULL);
+
+      ca_proplist_destroy (p);
+
+      if (res != CA_SUCCESS && res != CA_ERROR_DISABLED)
+        {      
+          /* ...and in case that failed we use the classic X11 bell. */
+          XkbForceDeviceBell (display->xdisplay, 
+                              xkb_bell_event->device, 
+                              xkb_bell_event->bell_class, 
+                              xkb_bell_event->bell_id, 
+                              xkb_bell_event->percent);
+        }
+    }
 }
 #endif /* HAVE_XKB */
 
-/**
- * Turns the bell to audible or visual. This tells X what to do, but
- * not Metacity; you will need to set the "visual bell" pref for that.
- *
- * If the configure script found we had no XKB, this is a no-op.
- *
- * \param display  The display we're configuring
- * \param audible  True for an audible bell, false for a visual bell
- */
 void
 meta_bell_set_audible (MetaDisplay *display, gboolean audible)
 {
-#ifdef HAVE_XKB
-  XkbChangeEnabledControls (display->xdisplay,
-			    XkbUseCoreKbd,
-			    XkbAudibleBellMask,
-			    audible ? XkbAudibleBellMask : 0);
-#endif  
 }
 
-/**
- * Initialises the bell subsystem. This involves intialising
- * XKB (which, despite being a keyboard extension, is the
- * place to look for bell notifications), then asking it
- * to send us bell notifications, and then also switching
- * off the audible bell if we're using a visual one ourselves.
- *
- * Unlike most X extensions we use, we only initialise XKB here
- * (rather than in main()). It's possible that XKB is not
- * installed at all, but if that was known at build time
- * we will have HAVE_XKB undefined, which will cause this
- * function to be a no-op.
- *
- * \param display  The display which is opening
- *
- * \bug There is a line of code that's never run that tells
- * XKB to reset the bell status after we quit. Bill H said
- * (<http://bugzilla.gnome.org/show_bug.cgi?id=99886#c12>)
- * that XFree86's implementation is broken so we shouldn't
- * call it, but that was in 2002. Is it working now?
- */
 gboolean
 meta_bell_init (MetaDisplay *display)
 {
@@ -363,8 +355,7 @@ meta_bell_init (MetaDisplay *display)
       XkbChangeEnabledControls (display->xdisplay,
 				XkbUseCoreKbd,
 				XkbAudibleBellMask,
-				meta_prefs_bell_is_audible () 
-				? XkbAudibleBellMask : 0);
+                                0);
       if (visual_bell_auto_reset) {
 	XkbSetAutoResetControls (display->xdisplay,
 				 XkbAudibleBellMask,
@@ -377,15 +368,6 @@ meta_bell_init (MetaDisplay *display)
   return FALSE;
 }
 
-/**
- * Shuts down the bell subsystem.
- *
- * \param display  The display which is closing
- *
- * \bug This is never called! If we had XkbSetAutoResetControls
- * enabled in meta_bell_init(), this wouldn't be a problem, but
- * we don't.
- */
 void
 meta_bell_shutdown (MetaDisplay *display)
 {
