@@ -23,7 +23,7 @@
  */
 
 #define _GNU_SOURCE
-#define _POSIX_C_SOURCE /* for fdopen() */
+#define _POSIX_C_SOURCE 200112L /* for fdopen() */
 
 #include <config.h>
 #include "util.h"
@@ -37,6 +37,8 @@
 #include <X11/Xlib.h>   /* must explicitly be included for Solaris; #326746 */
 #include <X11/Xutil.h>  /* Just for the definition of the various gravities */
 
+MetaNexus *sigchld_nexus;
+
 #ifdef HAVE_BACKTRACE
 #include <execinfo.h>
 void
@@ -537,3 +539,129 @@ meta_gravity_to_string (int gravity)
       break;
     }
 }
+
+GPid
+meta_show_dialog (const char *type,
+                  const char *message,
+                  const char *timeout,
+                  const gint screen_number,
+                  const char *ok_text,
+                  const char *cancel_text,
+                  const int transient_for,
+                  GSList *columns,
+                  GSList *entries)
+{
+  GError *error = NULL;
+  char *screen_number_text = g_strdup_printf("%d", screen_number);
+  GSList *tmp;
+  int i=0;
+  GPid child_pid;
+  const char **argvl = g_malloc(sizeof (char*) *
+                                (15 +
+                                 g_slist_length (columns)*2 +
+                                 g_slist_length (entries)));
+
+  argvl[i++] = "zenity";
+  argvl[i++] = type;
+  argvl[i++] = "--screen";
+  argvl[i++] = screen_number_text;
+  argvl[i++] = "--title";
+  /* Translators: This is the title used on dialog boxes */
+  argvl[i++] = _("Metacity");
+  argvl[i++] = "--text";
+  argvl[i++] = message;
+  
+  if (timeout)
+    {
+      argvl[i++] = "--timeout";
+      argvl[i++] = timeout;
+    }
+
+  if (ok_text)
+    {
+      argvl[i++] = "--ok-label";
+      argvl[i++] = ok_text;
+     }
+
+  if (cancel_text)
+    {
+      argvl[i++] = "--cancel-label";
+      argvl[i++] = cancel_text;
+    }
+  
+  tmp = columns;
+  while (tmp)
+    {
+      argvl[i++] = "--column";
+      argvl[i++] = tmp->data;
+      tmp = tmp->next;
+    }
+
+  tmp = entries;
+  while (tmp)
+    {
+      argvl[i++] = tmp->data;
+      tmp = tmp->next;
+    }
+    
+  argvl[i] = NULL;
+
+  if (transient_for)
+    {
+      gchar *env = g_strdup_printf("%d", transient_for);
+      setenv ("WINDOWID", env, 1);
+      g_free (env);
+    }
+
+  g_spawn_async (
+                 "/",
+                 (gchar**) argvl, /* ugh */
+                 NULL,
+                 G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
+                 NULL, NULL,
+                 &child_pid,
+                 &error
+                 );
+
+  if (transient_for)
+    unsetenv ("WINDOWID");
+
+  g_free (argvl);
+  g_free (screen_number_text);
+
+  if (error)
+    {
+      meta_warning ("%s\n", error->message);
+      g_error_free (error);
+    }
+
+  return child_pid;
+}
+
+GType
+meta_nexus_get_type (void)
+{
+  static GType nexus_type = 0;
+
+  if (!nexus_type)
+    {
+      static const GTypeInfo nexus_info =
+      {
+        sizeof (MetaNexusClass),
+	NULL, NULL, NULL, NULL, NULL,
+	sizeof (MetaNexus),
+	0,
+	NULL, NULL
+      };
+
+      nexus_type = g_type_register_static (G_TYPE_OBJECT,
+					   "MetaNexus",
+					   &nexus_info,
+					   0);
+    }
+
+  return nexus_type;
+}
+
+/* eof util.c */
+
